@@ -24,7 +24,8 @@ from oslo_policy import opts as oslo_policy_opts
 from oslo_service import service as os_service
 
 from searchlight.common import utils
-from searchlight.i18n import _LE, _LI
+from searchlight.i18n import _
+from searchlight.pipeline import PipelineManager
 
 LOG = logging.getLogger(__name__)
 
@@ -47,10 +48,11 @@ CONF.register_opts(listener_opts, group="listener")
 
 class NotificationEndpoint(object):
 
-    def __init__(self, plugins):
+    def __init__(self, plugins, pipeline_manager):
         self.plugins = plugins
+        self.pipeline_manager = pipeline_manager
         self.notification_target_map = {}
-        for plugin_type, plugin in six.iteritems(self.plugins):
+        for plugin_type, plugin in self.plugins.items():
             try:
                 handler = plugin.obj.get_notification_handler()
                 if not handler:
@@ -64,13 +66,13 @@ class NotificationEndpoint(object):
                     self.notification_target_map.setdefault(
                         event.lower(), []).append(plugin.obj)
             except Exception as e:
-                LOG.error(_LE("Failed to retrieve supported notification"
-                              " events from search plugins "
-                              "%(ext)s: %(e)s") %
+                LOG.error("Failed to retrieve supported notification"
+                          " events from search plugins "
+                          "%(ext)s: %(e)s" %
                           {'ext': plugin.name, 'e': e})
 
     def _log_notification(self, handler, ctxt, doc_type, event_type,
-                          payload, metadata):
+                          payload, metadata, priority):
         project = ctxt.get('project_id', ctxt.get('tenant_id',
                                                   ctxt.get('tenant', '-')))
         if not project:
@@ -80,21 +82,24 @@ class NotificationEndpoint(object):
         log_context = {'event_type': event_type,
                        'doc_type': doc_type,
                        'timestamp': metadata['timestamp'],
-                       'project': project}
+                       'project': project,
+                       'priority': priority}
         payload_fields = handler.get_log_fields(event_type, payload)
         additional = " ".join("%s:%s" % (k, v or '-')
                               for k, v in payload_fields)
         log_context['additional'] = additional or ''
-        LOG.info(_LI("Starting %(doc_type)s %(event_type)s \"%(timestamp)s\" "
-                     "project_id:%(project)s %(additional)s"), log_context)
+        LOG.info("Starting %(doc_type)s %(event_type)s \"%(timestamp)s\" "
+                 "priority:%(priority)s project_id:%(project)s "
+                 "%(additional)s", log_context)
         return log_context
 
     def _log_finished(self, log_context):
-        LOG.info(_LI("Finished %(doc_type)s %(event_type)s \"%(timestamp)s\" "
-                     "project_id:%(project)s %(additional)s"), log_context)
-
-    def info(self, ctxt, publisher_id, event_type, payload, metadata):
+        LOG.info("Finished %(doc_type)s %(event_type)s \"%(timestamp)s\" "
+                 "priority:%(priority)s project_id:%(project)s "
+                 "%(additional)s", log_context)
 
+    def _process_event(self, ctxt, publisher_id, event_type, payload,
+                       metadata, priority):
         event_type_l = event_type.lower()
         # The notification map contains a list of plugins for each event
         # type subscribed to
@@ -102,15 +107,27 @@ class NotificationEndpoint(object):
             handler = plugin.get_notification_handler()
             log_context = self._log_notification(
                 handler, ctxt, plugin.document_type,
-                event_type_l, payload, metadata)
-            handler.process(
+                event_type_l, payload, metadata, priority)
+            items = handler.process(
                 ctxt,
                 publisher_id,
                 event_type,
                 payload,
                 metadata)
+            # TODO(lei-zh): Add error handing and message acknowledgement
+            # Publishers only work for notification updates
+            if items:
+                self.pipeline_manager.publish(items)
             self._log_finished(log_context)
 
+    def info(self, ctxt, publisher_id, event_type, payload, metadata):
+        self._process_event(ctxt, publisher_id, event_type, payload, metadata,
+                            'INFO')
+
+    def error(self, ctxt, publisher_id, event_type, payload, metadata):
+        self._process_event(ctxt, publisher_id, event_type, payload, metadata,
+                            'ERROR')
+
 
 class ListenerService(os_service.Service):
     def __init__(self, *args, **kwargs):
@@ -121,33 +138,36 @@ class ListenerService(os_service.Service):
 
     def topics_and_exchanges(self):
         topics_exchanges = set()
-        for plugin_type, plugin in six.iteritems(self.plugins):
+        for plugin_type, plugin in self.plugins.items():
             try:
                 handler = plugin.obj.get_notification_handler()
-                topic_exchanges = handler.get_notification_topics_exchanges()
-                for plugin_topic in topic_exchanges:
-                    if isinstance(plugin_topic, six.string_types):
-                        raise Exception(
-                            _LE("Plugin %s should return a list of topic"
-                                "exchange pairs") % plugin.__class__.__name__)
-                    topics_exchanges.add(plugin_topic)
+                if handler:
+                    topic_exchanges = (
+                        handler.get_notification_topics_exchanges())
+                    for plugin_topic in topic_exchanges:
+                        if isinstance(plugin_topic, six.string_types):
+                            raise Exception(
+                                _("Plugin %s should return a list of topic"
+                                  "exchange pairs") %
+                                plugin.__class__.__name__)
+                        topics_exchanges.add(plugin_topic)
             except Exception as e:
-                LOG.error(_LE("Failed to retrieve notification topic(s)"
-                              " and exchanges from search plugin "
-                              "%(ext)s: %(e)s") %
+                LOG.error("Failed to retrieve notification topic(s)"
+                          " and exchanges from search plugin "
+                          "%(ext)s: %(e)s" %
                           {'ext': plugin.name, 'e': e})
 
         return topics_exchanges
 
     def start(self):
         super(ListenerService, self).start()
-        transport = oslo_messaging.get_transport(CONF)
+        transport = oslo_messaging.get_notification_transport(CONF)
         targets = [
             oslo_messaging.Target(topic=pl_topic, exchange=pl_exchange)
             for pl_topic, pl_exchange in self.topics_exchanges_set
         ]
         endpoints = [
-            NotificationEndpoint(self.plugins)
+            NotificationEndpoint(self.plugins, PipelineManager(self.plugins))
         ]
         listener = oslo_messaging.get_notification_listener(
             transport,
